#!/usr/bin/ruby -w
require 'Korundum'
require 'thread'

# Check interval in minutes
CHECK_INTERVAL = 30

NAME = "Yum Update Notifier"

class MySystemTray < KDE::SystemTray
  slots 'quit()',
	'checkUpdateFinished(KProcess*)',
	'checkForUpdates()',
        'receiveYumOutput(KProcess*,char*,int)',
        'showUpdates()',
        'startUpdate()',
        'updateFinished(KProcess*)',
        'showYumOutput()'

  def initialize() 
      super(nil,"")
      setCaption(NAME)
      icons = KDE::IconLoader.new()
      @okIcon = icons.loadIcon("button_ok", KDE::Icon::Toolbar)
      @updateIcon = icons.loadIcon("bell", KDE::Icon::Toolbar)
      @checkIcon = icons.loadIcon("help", KDE::Icon::Toolbar)
      @execIcon = icons.loadIcon("exec", KDE::Icon::Toolbar)
      @updateIntervall = CHECK_INTERVAL*60000
      @showUpdatesAction = contextMenu().insertItem("Available Updates")
      contextMenu().connectItem(@showUpdatesAction, 
              self, SLOT('showUpdates()'))

      @startUpdateAction = contextMenu().insertItem("Start Update")
      contextMenu().connectItem(@startUpdateAction,
              self, SLOT('startUpdate()'))
 
      @showYumOutputAction = contextMenu().insertItem("Show Yum Output")
      contextMenu().connectItem(@showYumOutputAction,
              self, SLOT('showYumOutput()'))
 
      createYumOutputDlg()

      enableUpdate(false)
      setPixmap(@checkIcon)
      connect(self, SIGNAL('quitSelected()'),
              self, SLOT('quit()'))
      checkForUpdates()
 end

  def createYumOutputDlg() 
     @yumOutputDlg = KDE::DialogBase::new(self,"Yum Output",false,NAME+" - Yum Output",KDE::DialogBase::Ok)
     @yumOutputDlgTextEdit = Qt::TextEdit::new(@yumOutputDlg)
     @yumOutputDlg.setMainWidget(@yumOutputDlgTextEdit)
     @yumOutputDlgTextEdit.setFamily("courier")
     @yumOutputDlg.resize(600,400)
  end

  def showYumOutput()
     @yumOutputDlg.show()
  end 

  def startUpdate() 
      setPixmap(@execIcon)
      puts 'Starting Update'
      proc = KDE::Process.new
      proc << "kdesu" << "-t" << "yum -y update"
      @yumoutput = ""
      connect(proc, SIGNAL('processExited(KProcess*)'),
              self, SLOT('updateFinished(KProcess*)'))
      connect(proc, SIGNAL('receivedStdout(KProcess*,char*,int)'),
              self, SLOT('receiveYumOutput(KProcess*,char*,int)'))
      proc.start(KDE::Process::NotifyOnExit, KDE::Process::Stdout)
  end

  def updateFinished(proc) 
      passivePopup("Update Finished")
      checkForUpdates()
  end

  def passivePopup(msg) 
      KDE::PassivePopup::message(NAME,msg,self)
  end
 
  def enableUpdate(b) 
      contextMenu().setItemEnabled(@showUpdatesAction,b)
      contextMenu().setItemEnabled(@startUpdateAction,b)
  end

  def quit() 
      exit()
  end

  def showUpdates() 
      dlg = KDE::DialogBase::new(self,"Yum Updates List",true,NAME+" - Updates",KDE::DialogBase::Ok)
      listBox = KDE::ListBox::new(dlg)
      @updates.each do |line|
         listBox.insertItem(line[0].to_s)
      end
#      layout.addWidget(listBox)
      dlg.setMainWidget(listBox)
      dlg.exec()    
  end

  def checkForUpdates() 
     checkUpdates()
     Qt::Timer.singleShot(@updateIntervall,self,SLOT('checkForUpdates()')) 
  end 

  def checkUpdates() 
      enableUpdate(false) 
      puts 'Checking for updates'
      Qt::ToolTip::add(self,"Checking for updates")
      setPixmap(@checkIcon)
      proc = KDE::Process.new
      proc << "yum" << "check-update"
      @yumoutput = ""
      connect(proc, SIGNAL('receivedStdout(KProcess*,char*,int)'),
	      self, SLOT('receiveYumOutput(KProcess*,char*,int)'))
      connect(proc, SIGNAL('processExited(KProcess*)'),
              self, SLOT('checkUpdateFinished(KProcess*)'))
      proc.start(KDE::Process::NotifyOnExit, KDE::Process::Stdout)
  end
  
  def receiveYumOutput(proc,buffer,buflen) 
      @yumoutput += buffer
      @yumOutputDlgTextEdit.append(buffer)
  end

  def checkUpdateFinished(proc) 
      puts 'Checking finished'

      if proc.normalExit()
         result = proc.exitStatus()
         puts 'Yum exit with status ' + result.to_s
         if result == 0
            puts 'No updates available'
            setPixmap(@okIcon)
            enableUpdate(false)
         elsif result == 100
            puts 'The following updates are available:'
            lines = []
            @yumoutput.each_line do |line|
                lines += [line]
            end
 
            empty = lines.index("\n") 
	    if empty != nil
                @updates = []
  	      	lines[empty+1..lines.length].each do |line|
                    splitted = line.split(' ')
        	    @updates += [splitted]
	      	end
            end
            passivePopup("New updates are available")
            Qt::ToolTip::add(self,"Updates are available.")
            setPixmap(@updateIcon)
            enableUpdate(true)
         else 
            puts 'Yum exit with unkown exit status ' + result.to_s
         end
      else
	 puts 'Yum did not exit normally.'
         setPixmap(@checkIcon)
      end
  end
end

about = KDE::AboutData.new("yumupkde",NAME, "0.1")
KDE::CmdLineArgs.init(ARGV, about)
a = KDE::UniqueApplication.new()
syst = MySystemTray.new()
syst.show

a.exec()

